/***************************************************************************
 *   Copyright (C) 2008 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "osd.h"
#include "defs.h"

#include <kglobal.h>
#include <kconfig.h>

#include <qapplication.h>
#include <qdesktopwidget.h>

OSD::OSD() : QWidget(0, "OSD", Qt::WType_TopLevel | Qt::WNoAutoErase | Qt::WStyle_Customize | Qt::WX11BypassWM | Qt::WStyle_StaysOnTop)
{
	this->setBackgroundMode(Qt::NoBackground);

	m_mixerImageOriginal = new QImage(MIXER_IMAGE);
	m_muteMixerImageOriginal = new QImage(MUTE_MIXER_IMAGE);
	m_mixerImage = new QImage();
	m_muteMixerImage = new QImage();

	m_painter = new QPainter();
	m_pixmapLabel = new QLabel(this, "PixmapLabel");
	m_textLabel = new QLabel(this, "TextLabel");
	m_volumeBar = new KProgress(this, "VolumeBar");

	m_backColor = new QColor();
	m_frontColor = new QColor();

	m_desktop = QApplication::desktop();
	connect(m_desktop, SIGNAL(resized(int)), this, SLOT(slotUpdatePosition(int)));

	m_timer = new QTimer(this, "Timer");
	connect(m_timer, SIGNAL(timeout()), this, SLOT(slotHide()));

	this->slotUpdate();
}


OSD::~OSD()
{
}


void OSD::slotUpdatePosition(int)
{
	KConfig *config = KGlobal::config();
	config->setGroup("Appearance");

	QRect rect = m_desktop->screenGeometry(m_desktop->primaryScreen());

	m_width = (unsigned int)(OSD_MIN_WIDTH + (OSD_MAX_WIDTH-OSD_MIN_WIDTH) / 100.0f * config->readNumEntry("Size"));
	m_height = (unsigned int)(m_width / OSD_ASPECT_RATIO);
	m_xPos = (unsigned int)((rect.width() - m_width) / 100.0f * config->readNumEntry("HPos"));
	m_yPos = (unsigned int)((rect.height() - m_height) / 100.0f * config->readNumEntry("VPos"));
	this->setGeometry(m_xPos, m_yPos, m_width, m_height);
}


void OSD::slotUpdate()
{
	this->slotUpdatePosition(0);

	KConfig *config = KGlobal::config();
	config->setGroup("Appearance");

	*m_backColor = config->readColorEntry("BackColor");
	*m_frontColor = config->readColorEntry("FrontColor");

	unsigned int pixmapX = (unsigned int)(m_width/100.0f*8.0f);
	unsigned int pixmapY = (unsigned int)(m_height/100.0f*7.0f);
	unsigned int pixmapSize = (unsigned int)(m_height/100.0f*55.0f);

	unsigned int textX = (int)(pixmapX + pixmapSize + (m_width/100.0f*3.0f));
	unsigned int textY = pixmapY;
	unsigned int textWidth = (unsigned int)(m_width - textX - pixmapX);
	unsigned int textHeight = pixmapSize;

	unsigned int volumeX = (unsigned int)(m_width/100.0f*8.0f);
	unsigned int volumeWidth = (unsigned int)(m_width - 2.0f*volumeX);
	unsigned int volumeHeight = (unsigned int)(m_height/100.0f*19.0f);
	unsigned int volumeY = (unsigned int)(m_height - m_height/100.0f*14.0f - volumeHeight);

	m_pixmapLabel->setGeometry(pixmapX, pixmapY, pixmapSize, pixmapSize);
	m_pixmapLabel->setPaletteBackgroundColor(*m_backColor);
	*m_mixerImage = m_mixerImageOriginal->smoothScale(pixmapSize, pixmapSize);
	*m_muteMixerImage = m_muteMixerImageOriginal->smoothScale(pixmapSize, pixmapSize);

	QFont font;
	font.setPixelSize((int)(m_height/100.0f*17.0f));
	font.setBold(true);
	m_textLabel->setGeometry(textX, textY, textWidth, textHeight);
	m_textLabel->setAlignment(Qt::AlignCenter);
	m_textLabel->setFont(font);
	m_textLabel->setPaletteForegroundColor(*m_frontColor);
	m_textLabel->setPaletteBackgroundColor(*m_backColor);

	font.setPixelSize((unsigned int)(volumeHeight/100.0f*50.0f));
	font.setBold(true);
	m_volumeBar->setFont(font);
	m_volumeBar->setGeometry(volumeX, volumeY, volumeWidth, volumeHeight);
	m_volumeBar->setPaletteForegroundColor(*m_frontColor);
	m_volumeBar->setPaletteBackgroundColor(*m_backColor);

	m_timeout = config->readNumEntry("Timeout");

}


void OSD::show(unsigned int volume, OSDType type)
{
	this->clearOSD();
	this->QWidget::show();

	if (type == Mute)
	{
		m_pixmapLabel->setPixmap(*m_muteMixerImage);
		m_volumeBar->setValue(0);
	}
	else
	{
		m_pixmapLabel->setPixmap(*m_mixerImage);
		m_volumeBar->setValue(volume);
	}

	switch (type)
	{
		case VolUp:
			m_textLabel->setText("Volume Up");
			break;
		case VolDown:
			m_textLabel->setText("Volume Down");
			break;
		case Mute:
			m_textLabel->setText("Mute");
			break;
		case Unmute:
			m_textLabel->setText("Unmute");
			break;
		default:
			m_textLabel->setText("Error!");
			break;	
	}

	m_pixmapLabel->show();
	m_textLabel->show();
	m_volumeBar->show();

	m_timer->start(m_timeout, true);
}


void OSD::clearOSD()
{
	m_pixmapLabel->hide();
	m_textLabel->hide();
	m_volumeBar->hide();
}


void OSD::slotHide()
{
	this->hide();
}


void OSD::paintEvent(QPaintEvent*)
{
	m_painter->begin(this);

	m_painter->setPen(QPainter::NoPen);
	m_painter->setBrush(*m_backColor);
	m_painter->drawRoundRect(0, 0, m_width, m_height, 18, 40);

	m_painter->setPen(*m_frontColor);
	m_painter->setBrush(QPainter::NoBrush);
	m_painter->drawRoundRect(0, 0, m_width, m_height, 18, 40);

	m_painter->end();
}


#include "osd.moc"


